from django.http import Http404
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.shortcuts import get_object_or_404
from django import VERSION

from calendar import month_name
from collections import defaultdict

from mezzanine.blog import views as blog_views
from mezzanine.utils.views import render
from mezzanine.blog.models import BlogCategory
from mezzanine.conf import settings
from mezzanine.generic.models import AssignedKeyword, Keyword
from mezzanine.utils.views import paginate

from .models import BlogProxy, BlogPost, Recipe


def blog_post_list_article(request, model=BlogPost, tag=None, year=None, month=None, username=None, category=None, template="blog/blog_post_list.html"):
    return blog_post_list(request=request, model=model, tag=tag, year=year, month=month, username=username, category=category, template=template)

def blog_post_list_recipe(request, model=Recipe, tag=None, year=None, month=None, username=None, category=None, template="recipe/recipe_list.html"):
    return blog_post_list(request=request, model=model, tag=tag, year=year, month=month, username=username, category=category, template=template)


def blog_post_list(request, model=BlogProxy, tag=None, year=None, month=None, username=None, category=None, template="blog/blog_post_list.html"):
    """
    Display a list of blog posts that are filtered by tag, year, month,
    author or category. Custom templates are checked for using the name
    ``blog/blog_post_list_XXX.html`` where ``XXX`` is either the
    category slug or author's username if given.
    """
    settings.use_editable()
    templates = []
    blog_posts = model.secondary.published(for_user=request.user)
    if tag is not None:
        tag = get_object_or_404(Keyword, slug=tag)
        blog_posts = blog_posts.filter(keywords__in=tag.assignments.all())
    if year is not None:
        blog_posts = blog_posts.filter(publish_date__year=year)
        if month is not None:
            blog_posts = blog_posts.filter(publish_date__month=month)
            month = month_name[int(month)]
    if category is not None:
        category = get_object_or_404(BlogCategory, slug=category)
        blog_posts = blog_posts.filter(categories=category)
        templates.append(u"blog/blog_post_list_%s.html" %
                         unicode(category.slug))
    author = None
    if username is not None:
        author = get_object_or_404(User, username=username)
        blog_posts = blog_posts.filter(user=author)
        templates.append(u"blog/blog_post_list_%s.html" % username)

    # We want to iterate keywords and categories for each blog post
    # without triggering "num posts x 2" queries.
    #
    # For Django 1.3 we create dicts mapping blog post IDs to lists of
    # categories and keywords, and assign these to attributes on each
    # blog post. The Blog model then uses accessor methods to retrieve
    # these attributes when assigned, which will fall back to the real
    # related managers for Django 1.4 and higher, which will already
    # have their data retrieved via prefetch_related.

    blog_posts = blog_posts.select_related("user")
    if VERSION >= (1, 4):
        blog_posts = blog_posts.prefetch_related("categories",
            "keywords__keyword")
    else:
        categories = defaultdict(list)
        if blog_posts:
            ids = ",".join([str(p.id) for p in blog_posts])
            for cat in BlogCategory.objects.raw(
                "SELECT * FROM blog_blogcategory "
                "JOIN blog_blogpost_categories "
                "ON blog_blogcategory.id = blogcategory_id "
                "WHERE blogpost_id IN (%s)" % ids):
                categories[cat.blogpost_id].append(cat)
        keywords = defaultdict(list)
        blogpost_type = ContentType.objects.get(app_label="blog",
            model="blogpost")
        assigned = AssignedKeyword.objects.filter(blogpost__in=blog_posts,
            content_type=blogpost_type).select_related("keyword")
        for a in assigned:
            keywords[a.object_pk].append(a.keyword)
        for i, post in enumerate(blog_posts):
            setattr(blog_posts[i], "_categories", categories[post.id])
            setattr(blog_posts[i], "_keywords", keywords[post.id])

    blog_posts = paginate(blog_posts, request.GET.get("page", 1),
        settings.BLOG_POST_PER_PAGE,
        settings.MAX_PAGING_LINKS)
    context = {"blog_posts": blog_posts, "year": year, "month": month,
               "tag": tag, "category": category, "author": author}
    templates.append(template)
    return render(request, templates, context)


def blog_post_detail(request, slug, year=None, month=None, day=None, template="blog_post_detail.html"):
    """. Custom templates are checked for using the name
    ``blog/blog_post_detail_XXX.html`` where ``XXX`` is the blog
    posts's slug.
    """

    blog_posts = BlogProxy.secondary.published(for_user=request.user).filter(slug=slug)
    if len(blog_posts) != 1:
        raise Http404(_('Slug %s does not exist.' % slug))
    blog_post = blog_posts[0]
    context = {"blog_post": blog_post, "editable_obj": blog_post}
    templates = [u"%sblog_post_detail_%s.html" % (blog_post.template_dir, unicode(slug)), u"%s%s" % (blog_post.template_dir, template)]
    return render(request, templates, context)


def blog_post_feed(request, format, **kwargs):
    return blog_views.blog_post_feed(request, format, **kwargs)






from mezzanine.twitter.templatetags.twitter_tags import tweets_for
from mezzanine.conf import settings
from mezzanine import template

register = template.Library()

@register.as_tag
def tweets_default(*args):
    """
    Tweets for the default settings.
    """
    settings.use_editable()
    query_type = settings.TWITTER_DEFAULT_QUERY_TYPE
    args = (settings.TWITTER_DEFAULT_QUERY,
            settings.TWITTER_DEFAULT_NUM_TWEETS)
    return tweets_for(query_type, args)



import re
import math

from django.conf.urls.defaults import url
from django.utils.translation import ugettext_lazy as _
from django.core.paginator import Paginator, InvalidPage
from django.db.models import Q
from django.http import Http404
from django.utils import simplejson

from mezzanine.conf import settings
from mezzanine.generic.models import ThreadedComment, AssignedKeyword, Keyword, Rating
from mezzanine.blog.models import BlogCategory
from mezzanine.core.models import CONTENT_STATUS_PUBLISHED
from mezzanine.utils.timezone import now

from tastypie import fields
from tastypie.paginator import Paginator as TastyPaginator
from tastypie.resources import ModelResource
from tastypie.throttle import CacheDBThrottle
from tastypie.utils import trailing_slash
from tastypie.serializers import Serializer
from tastypie.authentication import ApiKeyAuthentication
from tastypie.authorization import DjangoAuthorization, ReadOnlyAuthorization
from tastypie.contrib.contenttypes.fields import GenericForeignKeyField

import html2text

from .models import BlogProxy, Recipe, BlogPost, Ingredient, WorkingHours, CookingTime, RestPeriod
from .fields import DIFFICULTIES, UNITS


class CamelCaseJSONSerializer(Serializer):
    formats = ['json']
    content_types = {
        'json': 'application/json',
    }

    def to_json(self, data, options=None):
        # Changes underscore_separated names to camelCase names to go from python convention to javacsript convention
        data = self.to_simple(data, options)

        def underscoreToCamel(match):
            return match.group()[0] + match.group()[2].upper()

        def camelize(data):
            if isinstance(data, dict):
                new_dict = {}
                for key, value in data.items():
                    new_key = re.sub(r"[a-z]_[a-z]", underscoreToCamel, key)
                    new_dict[new_key] = camelize(value)
                return new_dict
            if isinstance(data, (list, tuple)):
                for i in range(len(data)):
                    data[i] = camelize(data[i])
                return data
            return data

        camelized_data = camelize(data)

        return simplejson.dumps(camelized_data, sort_keys=True)

    def from_json(self, content):
        # Changes camelCase names to underscore_separated names to go from javascript convention to python convention
        data = simplejson.loads(content)

        def camelToUnderscore(match):
            return match.group()[0] + "_" + match.group()[1].lower()

        def underscorize(data):
            if isinstance(data, dict):
                new_dict = {}
                for key, value in data.items():
                    new_key = re.sub(r"[a-z][A-Z]", camelToUnderscore, key)
                    new_dict[new_key] = underscorize(value)
                return new_dict
            if isinstance(data, (list, tuple)):
                for i in range(len(data)):
                    data[i] = underscorize(data[i])
                return data
            return data

        underscored_data = underscorize(data)

        return underscored_data



class RestKitPaginator(TastyPaginator):
    def page(self):
        output = super(RestKitPaginator, self).page()
        output['meta']['pageNumber'] = int(output['meta']['offset'] / output['meta']['limit']) + 1 if self.limit > 0 else 1
        output['meta']['totalPages'] = math.ceil(float(output['meta']['total_count']) / output['meta']['limit']) if self.limit > 0 else 1
        return output



class CategoryResource(ModelResource):
    posts = fields.ToManyField('mezzanine_recipes.api.PostResource', 'blogposts')

    class Meta:
        queryset = BlogCategory.objects.all()
        resource_name = "categories"
        fields = ['id', 'title',]
        list_allowed_methods = ['get',]
        detail_allowed_methods = ['get',]
        limit = 0
        throttle = CacheDBThrottle()
        serializer = CamelCaseJSONSerializer()
        authentication = ApiKeyAuthentication()
        authorization = ReadOnlyAuthorization()

    def get_object_list(self, request, *args, **kwargs):
        return BlogCategory.objects.filter(Q(blogposts__publish_date__lte=now()) | Q(blogposts__publish_date__isnull=True),
                                           Q(blogposts__expiry_date__gte=now()) | Q(blogposts__expiry_date__isnull=True),
                                           Q(blogposts__status=CONTENT_STATUS_PUBLISHED)).distinct()

    def alter_list_data_to_serialize(self, request, data):
        data['categories'] = data['objects']
        del data['objects']
        return data

    def alter_deserialized_list_data(self, request, data):
        data['objects'] = data['categories']
        del data['categories']
        return data



class BlogPostResource(ModelResource):
    categories = fields.ToManyField('mezzanine_recipes.api.CategoryResource', 'categories', full=True)

    class Meta:
        queryset = BlogPost.secondary.published().order_by('-publish_date')
        resource_name = "blog"
        fields = ['id', 'title', 'featured_image', 'description', 'publish_date', 'allow_comments', 'comments_count', 'rating_average', 'rating_count', 'modified_date',]
        list_allowed_methods = ['get',]
        detail_allowed_methods = ['get',]
        throttle = CacheDBThrottle()
        filtering = {
            "publish_date": ('gt',),
        }
        serializer = CamelCaseJSONSerializer()
        authentication = ApiKeyAuthentication()
        authorization = ReadOnlyAuthorization()
        limit = 5
        paginator_class = RestKitPaginator

    def dehydrate_featured_image(self, bundle):
        if bundle.data['featured_image']:
            return settings.MEDIA_URL+bundle.data['featured_image']
        else:
            return None

    def prepend_urls(self):
        return [
            url(r"^(?P<resource_name>%s)/search%s$" % (self._meta.resource_name, trailing_slash()), self.wrap_view('get_search'), name="api_get_search"),
        ]

    def alter_list_data_to_serialize(self, request, data):
        data['blogs'] = data['objects']
        del data['objects']
        return data

    def alter_deserialized_list_data(self, request, data):
        data['objects'] = data['blogs']
        del data['blogs']
        return data

    def get_search(self, request, **kwargs):
        self.method_check(request, allowed=['get'])
        self.is_authenticated(request)
        self.throttle_check(request)

        sqs = BlogPost.objects.search(request.GET.get('q', ''))
        paginator = Paginator(sqs, 20)

        try:
            page = paginator.page(int(request.GET.get('page', 1)))
        except InvalidPage:
            raise Http404(_("Sorry, no results on that page."))

        objects = []

        for result in page.object_list:
            bundle = self.build_bundle(obj=result, request=request)
            bundle = self.full_dehydrate(bundle)
            objects.append(bundle)

        object_list = {
            'objects': objects,
            }

        self.log_throttled_access(request)
        return self.create_response(request, object_list)



class RecipeResource(ModelResource):
    categories = fields.ToManyField('mezzanine_recipes.api.CategoryResource', 'categories', full=True)
    ingredients = fields.ToManyField('mezzanine_recipes.api.IngredientResource', 'ingredients', full=True)
    working_hours = fields.ToOneField('mezzanine_recipes.api.WorkingHoursResource', 'working_hours', full=True, null=True)
    cooking_time = fields.ToOneField('mezzanine_recipes.api.CookingTimeResource', 'cooking_time', full=True, null=True)
    rest_period = fields.ToOneField('mezzanine_recipes.api.RestPeriodResource', 'rest_period', full=True, null=True)

    class Meta:
        queryset = Recipe.secondary.published().order_by('-publish_date')
        resource_name = "recipe"
        fields = ['id', 'title', 'featured_image', 'summary', 'content', 'portions', 'difficulty', 'publish_date', 'allow_comments', 'comments_count', 'rating_average', 'rating_count', 'modified_date',]
        list_allowed_methods = ['get',]
        detail_allowed_methods = ['get',]
        throttle = CacheDBThrottle()
        filtering = {
            "publish_date": ('gt',),
        }
        serializer = CamelCaseJSONSerializer()
        authentication = ApiKeyAuthentication()
        authorization = ReadOnlyAuthorization()
        limit = 5
        paginator_class = RestKitPaginator

    def dehydrate_featured_image(self, bundle):
        if bundle.data['featured_image']:
            return settings.MEDIA_URL+bundle.data['featured_image']
        else:
            return None

    def dehydrate_difficulty(self, bundle):
        if bundle.data['difficulty']:
            return dict(DIFFICULTIES)[bundle.data['difficulty']]
        else:
            return ""

    def dehydrate(self, bundle):
        content = bundle.data['content']
        bundle.data['description'] = re.sub("\n+$", "", re.sub(" +\* ", "- ", html2text.html2text(content)))
        del bundle.data['content']
        return bundle

    def prepend_urls(self):
        return [
            url(r"^(?P<resource_name>%s)/search%s$" % (self._meta.resource_name, trailing_slash()), self.wrap_view('get_search'), name="api_get_search"),
        ]

    def alter_list_data_to_serialize(self, request, data):
        data['recipes'] = data['objects']
        del data['objects']
        return data

    def alter_deserialized_list_data(self, request, data):
        data['objects'] = data['recipes']
        del data['recipes']
        return data

    def get_search(self, request, **kwargs):
        self.method_check(request, allowed=['get'])
        self.is_authenticated(request)
        self.throttle_check(request)

        sqs = Recipe.objects.search(request.GET.get('q', ''))
        paginator = Paginator(sqs, 20)

        try:
            page = paginator.page(int(request.GET.get('page', 1)))
        except InvalidPage:
            raise Http404(_("Sorry, no results on that page."))

        objects = []

        for result in page.object_list:
            bundle = self.build_bundle(obj=result, request=request)
            bundle = self.full_dehydrate(bundle)
            objects.append(bundle)

        object_list = {
            'objects': objects,
            }

        self.log_throttled_access(request)
        return self.create_response(request, object_list)



class PostResource(ModelResource):
    categories = fields.ToManyField('mezzanine_recipes.api.CategoryResource', 'categories', full=True)

    class Meta:
        queryset = BlogProxy.secondary.published().order_by('-publish_date')
        resource_name = "post"
        fields = ['id', 'title', 'featured_image', 'description', 'publish_date', 'allow_comments', 'comments_count', 'rating_average', 'rating_count', 'modified_date',]
        list_allowed_methods = ['get',]
        detail_allowed_methods = ['get',]
        throttle = CacheDBThrottle()
        filtering = {
            "publish_date": ('gt',),
        }
        serializer = CamelCaseJSONSerializer()
        authentication = ApiKeyAuthentication()
        authorization = ReadOnlyAuthorization()
        limit = 5
        paginator_class = RestKitPaginator

    def dehydrate_featured_image(self, bundle):
        if bundle.data['featured_image']:
            return settings.MEDIA_URL+bundle.data['featured_image']
        else:
            return None

    def dehydrate(self, bundle):
        if isinstance(bundle.obj, BlogPost):
            blog_res = BlogPostResource()
            rr_bundle = blog_res.build_bundle(obj=bundle.obj, request=bundle.request)
            bundle.data = blog_res.full_dehydrate(rr_bundle).data
        elif isinstance(bundle.obj, Recipe):
            recipe_res = RecipeResource()
            br_bundle = recipe_res.build_bundle(obj=bundle.obj, request=bundle.request)
            bundle.data = recipe_res.full_dehydrate(br_bundle).data
        return bundle

    def prepend_urls(self):
        return [
            url(r"^(?P<resource_name>%s)/search%s$" % (self._meta.resource_name, trailing_slash()), self.wrap_view('get_search'), name="api_get_search"),
        ]

    def alter_list_data_to_serialize(self, request, data):
        data['posts'] = data['objects']
        del data['objects']
        return data

    def alter_deserialized_list_data(self, request, data):
        data['objects'] = data['posts']
        del data['posts']
        return data

    def get_search(self, request, **kwargs):
        self.method_check(request, allowed=['get'])
        self.is_authenticated(request)
        self.throttle_check(request)

        sqs = BlogProxy.secondary.search(request.GET.get('q', ''))
        paginator = Paginator(sqs, 20)

        try:
            page = paginator.page(int(request.GET.get('page', 1)))
        except InvalidPage:
            raise Http404(_("Sorry, no results on that page."))

        objects = []

        for result in page.object_list:
            bundle = self.build_bundle(obj=result, request=request)
            bundle = self.full_dehydrate(bundle)
            objects.append(bundle)

        object_list = {
            'objects': objects,
            }

        self.log_throttled_access(request)
        return self.create_response(request, object_list)



class CommentResource(ModelResource):
    replied_to = fields.ToOneField('mezzanine_recipes.api.CommentResource', 'replied_to', null=True)
    content_object = GenericForeignKeyField({
        BlogProxy: PostResource,
        BlogPost: BlogPostResource,
        Recipe: RecipeResource
    }, 'content_object')

    class Meta:
        queryset = ThreadedComment.objects.visible()
        resource_name = "comments"
        fields = ['id', 'object_pk', 'comment', 'submit_date', 'user_name', 'user_email', 'user_url', 'replied_to',]
        list_allowed_methods = ['get', 'post',]
        detail_allowed_methods = ['get',]
        throttle = CacheDBThrottle()
        filtering = {
            'object_pk': ('exact',),
            }
        serializer = CamelCaseJSONSerializer()
        authentication = ApiKeyAuthentication()
        authorization = DjangoAuthorization()
        limit = 0

    def dehydrate_user_email(self, bundle):
        return None

    def dehydrate_user_url(self, bundle):
        return None

    def alter_list_data_to_serialize(self, request, data):
        data['comments'] = data['objects']
        del data['objects']
        return data

    def alter_deserialized_list_data(self, request, data):
        data['objects'] = data['comments']
        del data['comments']
        return data



class RatingResource(ModelResource):
    post = GenericForeignKeyField({
        BlogProxy: PostResource,
        BlogPost: BlogPostResource,
        Recipe: RecipeResource
    }, 'content_object')

    class Meta:
        queryset = Rating.objects.all()
        resource_name = "rating"
        fields = ['id', 'object_pk', 'value',]
        list_allowed_methods = ['get', 'post',]
        detail_allowed_methods = ['get',]
        throttle = CacheDBThrottle()
        filtering = {
            'object_pk': ('exact',),
            }
        serializer = CamelCaseJSONSerializer()
        authentication = ApiKeyAuthentication()
        authorization = DjangoAuthorization()
        limit = 0

    def alter_list_data_to_serialize(self, request, data):
        data['ratings'] = data['objects']
        del data['objects']
        return data

    def alter_deserialized_list_data(self, request, data):
        data['objects'] = data['ratings']
        del data['ratings']
        return data



class KeywordResource(ModelResource):
    keyword = fields.ToManyField('mezzanine_recipes.api.AssignedKeywordResource', 'assignments')

    class Meta:
        queryset = Keyword.objects.all()
        resource_name = "keywords"
        fields = ['id', 'title']
        list_allowed_methods = ['get',]
        detail_allowed_methods = ['get',]
        limit = 0
        throttle = CacheDBThrottle()
        filtering = {
            'title': ('exact',),
            }
        serializer = CamelCaseJSONSerializer()
        authentication = ApiKeyAuthentication()
        authorization = ReadOnlyAuthorization()

    def alter_list_data_to_serialize(self, request, data):
        data['keywords'] = data['objects']
        del data['objects']
        return data

    def alter_deserialized_list_data(self, request, data):
        data['objects'] = data['keywords']
        del data['keywords']
        return data



class AssignedKeywordResource(ModelResource):
    assignments = fields.ToOneField('mezzanine_recipes.api.KeywordResource', 'keyword', full=True)
    post = GenericForeignKeyField({
        BlogProxy: PostResource,
        BlogPost: BlogPostResource,
        Recipe: RecipeResource
    }, 'content_object')

    class Meta:
        queryset = AssignedKeyword.objects.all()
        resource_name = "assigned_keywords"
        fields = ['id', 'object_pk', '_order',]
        list_allowed_methods = ['get',]
        detail_allowed_methods = ['get',]
        throttle = CacheDBThrottle()
        filtering = {
            'object_pk': ('exact',),
            }
        serializer = CamelCaseJSONSerializer()
        authentication = ApiKeyAuthentication()
        authorization = ReadOnlyAuthorization()
        limit = 0

    def alter_list_data_to_serialize(self, request, data):
        data['assignedKeywords'] = data['objects']
        del data['objects']
        return data

    def alter_deserialized_list_data(self, request, data):
        data['objects'] = data['assignedKeywords']
        del data['assignedKeywords']
        return data



class IngredientResource(ModelResource):
    recipe = fields.ToOneField('mezzanine_recipes.api.RecipeResource', 'recipe')

    class Meta:
        queryset = Ingredient.objects.all()
        resource_name = "ingredient"
        fields = ['id', 'quantity', 'unit', 'ingredient', 'note',]
        list_allowed_methods = ['get',]
        detail_allowed_methods = ['get',]
        limit = 0
        throttle = CacheDBThrottle()
        serializer = CamelCaseJSONSerializer()
        authentication = ApiKeyAuthentication()
        authorization = ReadOnlyAuthorization()

    def get_object_list(self, request, *args, **kwargs):
        return Ingredient.objects.filter(Q(recipe__publish_date__lte=now()) | Q(recipe__publish_date__isnull=True),
                                         Q(recipe__expiry_date__gte=now()) | Q(recipe__expiry_date__isnull=True),
                                         Q(recipe__status=CONTENT_STATUS_PUBLISHED))

    def dehydrate_unit(self, bundle):
        if bundle.data['unit']:
            return dict(UNITS)[bundle.data['unit']]
        else:
            return ""

    def alter_list_data_to_serialize(self, request, data):
        data['ingredients'] = data['objects']
        del data['objects']
        return data

    def alter_deserialized_list_data(self, request, data):
        data['objects'] = data['ingredients']
        del data['ingredients']
        return data



class WorkingHoursResource(ModelResource):
    recipe = fields.ToOneField('mezzanine_recipes.api.RecipeResource', 'recipe')

    class Meta:
        queryset = WorkingHours.objects.all()
        resource_name = "working_hours"
        fields = ['id', 'hours', 'minutes',]
        list_allowed_methods = ['get',]
        detail_allowed_methods = ['get',]
        throttle = CacheDBThrottle()
        serializer = CamelCaseJSONSerializer()
        authentication = ApiKeyAuthentication()
        authorization = ReadOnlyAuthorization()

    def get_object_list(self, request, *args, **kwargs):
        return WorkingHours.objects.filter(Q(recipe__publish_date__lte=now()) | Q(recipe__publish_date__isnull=True),
                                           Q(recipe__expiry_date__gte=now()) | Q(recipe__expiry_date__isnull=True),
                                           Q(recipe__status=CONTENT_STATUS_PUBLISHED))

    def alter_list_data_to_serialize(self, request, data):
        data['workingHours'] = data['objects']
        del data['objects']
        return data

    def alter_deserialized_list_data(self, request, data):
        data['objects'] = data['workingHours']
        del data['workingHours']
        return data



class CookingTimeResource(ModelResource):
    recipe = fields.ToOneField('mezzanine_recipes.api.RecipeResource', 'recipe')

    class Meta:
        queryset = CookingTime.objects.all()
        resource_name = "cooking_time"
        fields = ['id', 'hours', 'minutes',]
        list_allowed_methods = ['get',]
        detail_allowed_methods = ['get',]
        throttle = CacheDBThrottle()
        serializer = CamelCaseJSONSerializer()
        authentication = ApiKeyAuthentication()
        authorization = ReadOnlyAuthorization()

    def get_object_list(self, request, *args, **kwargs):
        return CookingTime.objects.filter(Q(recipe__publish_date__lte=now()) | Q(recipe__publish_date__isnull=True),
                                          Q(recipe__expiry_date__gte=now()) | Q(recipe__expiry_date__isnull=True),
                                          Q(recipe__status=CONTENT_STATUS_PUBLISHED))

    def alter_list_data_to_serialize(self, request, data):
        data['cookingTimes'] = data['objects']
        del data['objects']
        return data

    def alter_deserialized_list_data(self, request, data):
        data['objects'] = data['cookingTimes']
        del data['cookingTimes']
        return data



class RestPeriodResource(ModelResource):
    recipe = fields.ToOneField('mezzanine_recipes.api.RecipeResource', 'recipe')

    class Meta:
        queryset = RestPeriod.objects.all()
        resource_name = "rest_period"
        fields = ['id', 'days', 'hours', 'minutes',]
        list_allowed_methods = ['get',]
        detail_allowed_methods = ['get',]
        throttle = CacheDBThrottle()
        serializer = CamelCaseJSONSerializer()
        authentication = ApiKeyAuthentication()
        authorization = ReadOnlyAuthorization()

    def get_object_list(self, request, *args, **kwargs):
        return RestPeriod.objects.filter(Q(recipe__publish_date__lte=now()) | Q(recipe__publish_date__isnull=True),
                                         Q(recipe__expiry_date__gte=now()) | Q(recipe__expiry_date__isnull=True),
                                         Q(recipe__status=CONTENT_STATUS_PUBLISHED))

    def alter_list_data_to_serialize(self, request, data):
        data['restPeriods'] = data['objects']
        del data['objects']
        return data

    def alter_deserialized_list_data(self, request, data):
        data['objects'] = data['restPeriods']
        del data['restPeriods']
        return data



from django.db import models
from django.db.models.query import QuerySet
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes.models import ContentType

from mezzanine.conf import settings
from mezzanine.core.models import Orderable
from mezzanine.core.managers import DisplayableManager
from mezzanine.blog.models import BlogPost as MezzanineBlogPost
from mezzanine.utils.timezone import now

from . import fields


class SubclassingQuerySet(QuerySet):

    def __getitem__(self, k):
        result = super(SubclassingQuerySet, self).__getitem__(k)
        if isinstance(result, BlogProxy) :
            return result.as_leaf_class()
        else :
            return result

    def __iter__(self):
        for item in super(SubclassingQuerySet, self).__iter__():
            yield item.as_leaf_class()


class BlogManager(DisplayableManager):
    def get_query_set(self):
        return SubclassingQuerySet(self.model)



class BlogProxy(MezzanineBlogPost):
    content_type = models.ForeignKey(ContentType,editable=False,null=True)
    modified_date = models.DateTimeField(_("Last Modified"), blank=True, null=True)

    template_dir = "blog/"
    secondary = BlogManager()

    def save(self, *args, **kwargs):
        self.modified_date = now()
        if not self.content_type:
            self.content_type = ContentType.objects.get_for_model(self.__class__)
        super(BlogProxy, self).save(*args, **kwargs)

    def as_leaf_class(self):
        content_type = self.content_type
        model = content_type.model_class()
        if model == BlogProxy:
            return self
        return model.objects.get(id=self.id)

    def _get_next_or_previous_by_publish_date(self, is_next, **kwargs):
        """
        Retrieves next or previous object by publish date. We implement
        our own version instead of Django's so we can hook into the
        published manager and concrete subclasses.
        """
        arg = "publish_date__gt" if is_next else "publish_date__lt"
        order = "publish_date" if is_next else "-publish_date"
        lookup = {arg: self.publish_date}
        concrete_model = self.__class__
        try:
            queryset = concrete_model.secondary.published
        except AttributeError:
            queryset = concrete_model.secondary.all
        try:
            return queryset(**kwargs).filter(**lookup).order_by(order)[0]
        except IndexError:
            pass


class BlogPost(BlogProxy):
    secondary = BlogManager()

    @models.permalink
    def get_absolute_url(self):
        url_name = "blog_post_detail"
        kwargs = {"slug": "%s/%s" % (settings.ARTICLES_SLUG, self.slug)}
        if settings.BLOG_URLS_USE_DATE:
            url_name = "blog_post_detail_date"
            month = str(self.publish_date.month)
            if len(month) == 1:
                month = "0" + month
            day = str(self.publish_date.day)
            if len(day) == 1:
                day = "0" + day
            kwargs.update({
                "day": day,
                "month": month,
                "year": self.publish_date.year,
                })
        return (url_name, (), kwargs)

    class Meta:
        verbose_name = _("Blog post")
        verbose_name_plural = _("Blog posts")
        ordering = ("-publish_date",)


class Recipe(BlogProxy):
    """
    Implements the recipe type of page with all recipe fields.
    """
    summary = models.TextField(_("Summary"), blank=True, null=True)
    portions = models.IntegerField(_("Portions"), blank=True, null=True)
    difficulty = models.IntegerField(_("Difficulty"), choices=fields.DIFFICULTIES, blank=True, null=True)
    source = models.URLField(_("Source"), blank=True, null=True, help_text=_("URL of the source recipe"))

    template_dir = "recipe/"
    secondary = BlogManager()
    search_fields = ("title", "summary", "description",)

    def __unicode__(self):
        return u'%s' % (self.title)

    @models.permalink
    def get_absolute_url(self):
        url_name = "blog_post_detail"
        kwargs = {"slug": "%s/%s" % (settings.RECIPES_SLUG, self.slug)}
        if settings.BLOG_URLS_USE_DATE:
            url_name = "blog_post_detail_date"
            month = str(self.publish_date.month)
            if len(month) == 1:
                month = "0" + month
            day = str(self.publish_date.day)
            if len(day) == 1:
                day = "0" + day
            kwargs.update({
                "day": day,
                "month": month,
                "year": self.publish_date.year,
                })
        return (url_name, (), kwargs)

    class Meta:
        verbose_name = _("Recipe")
        verbose_name_plural = _("Recipes")
        ordering = ("-publish_date",)


class Ingredient(Orderable):
    """
    Provides ingredient fields for managing recipe content and making
    it searchable.
    """
    recipe = models.ForeignKey("Recipe", verbose_name=_("Recipe"), related_name="ingredients")
    quantity = models.CharField(_("Quantity"), max_length=10, blank=True, null=True)
    unit = models.IntegerField(_("Unit"), choices=fields.UNITS, blank=True, null=True)
    ingredient = models.CharField(_("Ingredient"), max_length=100)
    note = models.CharField(_("Note"), max_length=200, blank=True, null=True)

    def __unicode__(self):
        _ingredient = u'%s' % (self.ingredient)

        if self.unit:
            _ingredient = u'%s %s' % (self.get_unit_display(), _ingredient)

        if self.quantity:
            _ingredient = u'%s %s' % (self.quantity, _ingredient)

        if len(self.note):
           _ingredient = u'%s - %s' % (_ingredient, self.note)

        return _ingredient

    class Meta:
        verbose_name = _("Ingredient")
        verbose_name_plural = _("Ingredients")


class Period(models.Model):
    """
    Provides fields for a period of time
    """
    hours = models.IntegerField(_("hours"), default=0)
    minutes = models.IntegerField(_("minutes"), default=0)

    def __unicode__(self):
        return "%02d:%02d" %(self.hours, self.minutes)

    class Meta:
        abstract = True


class WorkingHours(Period):
    """
    Provides working hour fields for cooking a recipe
    """
    recipe = models.OneToOneField("Recipe", verbose_name=_("Recipe"), related_name="working_hours")

    class Meta:
        verbose_name = _("working hour")
        verbose_name_plural = verbose_name


class CookingTime(Period):
    """
    Provides cooking time fields for cooking a recipe
    """
    recipe = models.OneToOneField("Recipe", verbose_name=_("Recipe"), related_name="cooking_time")

    class Meta:
        verbose_name = _("cooking time")
        verbose_name_plural = verbose_name


class RestPeriod(Period):
    """
    Provides rest time fields for cooking a recipe
    """
    recipe = models.OneToOneField("Recipe", verbose_name=_("Recipe"), related_name="rest_period")
    days = models.IntegerField(_("days"), default=0)

    def __unicode__(self):
        period = super(RestPeriod, self).__unicode__()
        return "%02d:%s" %(self.days, period)

    class Meta:
        verbose_name = _("rest period")
        verbose_name_plural = verbose_name



from copy import deepcopy

from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.models import User
#from django.conf import settings

from mezzanine.blog.admin import BlogPostAdmin as MezzanineBlogPostAdmin
from mezzanine.blog.models import BlogPost as MezzanineBlogPost
from mezzanine.core.admin import TabularDynamicInlineAdmin
from mezzanine.conf import settings

from tastypie.admin import ApiKeyInline
from tastypie.models import ApiAccess

from .models import Recipe, Ingredient, WorkingHours, CookingTime, RestPeriod, BlogPost


blogpost_fieldsets = deepcopy(MezzanineBlogPostAdmin.fieldsets)
blogpost2_fieldsets = deepcopy(MezzanineBlogPostAdmin.fieldsets)
blogpost_fieldsets[0][1]["fields"].extend(["summary", "portions", "difficulty", "source"])
blogpost2_fieldsets[0][1]["fields"].extend(["modified_date"])
recipe_list_display = deepcopy(MezzanineBlogPostAdmin.list_display)

if not settings.BLOG_USE_FEATURED_IMAGE:
    blogpost_fieldsets[0][1]["fields"].insert(-6, "featured_image")
    recipe_list_display.insert(0, "admin_thumb")

class WorkingHoursInline(admin.TabularInline):
    model = WorkingHours

class CookingTimeInline(admin.TabularInline):
    model = CookingTime

class RestPeriodInline(admin.TabularInline):
    model = RestPeriod
    fields = ("days", "hours", "minutes",)

class IngredientInline(TabularDynamicInlineAdmin):
    model = Ingredient

class RecipeAdmin(MezzanineBlogPostAdmin):
    """
    Admin class for recipes.
    """
    inlines = (IngredientInline, WorkingHoursInline, CookingTimeInline, RestPeriodInline,)
    fieldsets = blogpost_fieldsets
    list_display = recipe_list_display

class BlogPostAdmin(MezzanineBlogPostAdmin):
    fieldsets = blogpost2_fieldsets


admin.site.unregister(MezzanineBlogPost)
admin.site.register(BlogPost, BlogPostAdmin)
admin.site.register(Recipe, RecipeAdmin)


class UserModelAdmin(UserAdmin):
    if 'tastypie' in settings.INSTALLED_APPS:
        inlines = UserAdmin.inlines + [ApiKeyInline]

if 'tastypie' in settings.INSTALLED_APPS:
    admin.site.register(ApiAccess)
    admin.site.unregister(User)
    admin.site.register(User, UserModelAdmin)


from setuptools import setup, find_packages
from mezzanine_recipes import __version__

setup(
    name="mezzanine-recipes",
    version=__version__,
    description='Recipe blog posts for Mezzanine CMS',
    long_description=open("README.rst").read(),
    author='Thomas Jetzinger',
    author_email='thomas@jetzinger.com',
    url='https://github.com/tjetzinger/mezzanine-recipes',
    zip_safe=False,
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Django==1.4.4',
        'Mezzanine==1.4.1',
        'django-tastypie==0.9.12',
        'South==0.7.6',
        'uuid==1.30',
        'html2text==3.200.3',
    ],
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        "Programming Language :: Python",
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



"""
Default settings for the ``mezzanine_recipe`` app. Each of these can be
overridden in your project's settings module, just like regular
Django settings. The ``editable`` argument for each controls whether
the setting is editable via Django's admin.

Thought should be given to how a setting is actually used before
making it editable, as it may be inappropriate - for example settings
that are only read during startup shouldn't be editable, since changing
them would require an application reload.
"""

from django.utils.translation import ugettext_lazy as _

from mezzanine.conf import register_setting

register_setting(
    name="ADMIN_MENU_ORDER",
    description=_("Controls the ordering and grouping of the admin menu."),
    editable=False,
    default=(
        (_("Content"), ("pages.Page", "mezzanine_recipes.BlogPost", "mezzanine_recipes.Recipe",
                        "generic.ThreadedComment", (_("Media Library"), "fb_browse"),)),
        (_("Site"), ("sites.Site", "redirects.Redirect", "conf.Setting")),
        (_("Users"), ("auth.User", "auth.Group",)),
        (_("REST Service"), ("tastypie.ApiKey", "tastypie.ApiAccess",)),
    ),
)

register_setting(
    name="RECIPES_SLUG",
    description=_("Slug of the page object for the blog."),
    editable=False,
    default="recipes",
)

register_setting(
    name="ARTICLES_SLUG",
    description=_("Slug of the page object for the blog."),
    editable=False,
    default="articles",
)

register_setting(
    name="SEARCH_MODEL_CHOICES",
    description=_("Sequence of models that will be provided by default as "
                  "choices in the search form. Each model should be in the format "
                  "``app_label.model_name``. Only models that subclass "
                  "``mezzanine.core.models.Displayable`` should be used."),
    editable=False,
    default=("mezzanine_recipes.recipe", "mezzanine_recipes.BlogPost"),
)


from django.utils.translation import ugettext_lazy as _

# Constants for all available difficulty types.
SIMPLE = 1
MEDIUM = 2
DIFFICULT = 3

# Names for all available difficulty types.
DIFFICULTIES = (
    (SIMPLE, _("simple")),
    (MEDIUM, _("medium")),
    (DIFFICULT, _("difficult")),
)


# Constants for all available unit types.
CUP = 11
BAG = 97
SHEET = 98
SHEETS = 179
BUNCH = 16
BUNDLE = 204
CL = 12
CM = 46
THICKNESS = 215
DL = 44
CAN = 166
CANS = 156
THIN = 100
BUCKET = 2
AFEW = 43
AFEWSTALKS = 30
TBS = 13
TBSH = 59
SOMETHING = 22
POSSIBLY = 120
EXTRA = 220
BOTTLE = 55
BOTTLES = 184
G = 3
GLASS = 31
GLASSES = 93
GREATCAN = 63
GREATBOTTLE = 124
GREATSHE = 21
GREAT = 213
GREATHE = 180
GREATIT = 42
HALF = 70
HANDFUL = 48
KG = 28
SMALLBUNCH = 127
SMALLCAN = 104
SMALLGLASS = 83
SMALLHEAD = 18
SMALLBOTTLE = 249
SMALLSHE = 24
SMALL = 168
SMALLHE = 52
SMALLIT = 131
TUBER = 153
HEAD = 19
BALL = 231
BALLS = 217
BALLS2 = 122
BOX = 74
HEADS = 77
CORN = 192
LITER = 165
MGREATSHE = 25
MGREATHE = 53
MGREATIT = 90
MORE = 227
MG = 228
ML = 4
MSP = 5
NB = 7
PAIR = 76
PACKAGE = 243
PCK = 169
PKT = 1
PLATE = 119
PORT = 72
PRISE = 106
PRISES = 238
RING = 130
RIB = 84
DISC = 32
DISCS = 252
SHOT = 126
TAPENADE = 78
SPLASH = 15
ROD = 149
STALK = 255
STALKS = 57
PIECE = 87
CULM = 47
CHALKBOARD = 218
CHALKBOARDS = 116
CUP2 = 224
CUPS = 29
PART = 146
TL = 14
HEAPEDTL = 58
LEVELTL = 164
POT = 175
DROP = 20
TUBE = 125
BAG2 = 141
LOTS = 136
FEW = 45
ROOT = 198
ROOTS = 161
CUBE = 89
TOE = 26
BRACE = 150

# Names for all available unit types.
UNITS = (
    (CUP, _("cup")),
    (BAG, _("bag")),
    (SHEET, _("sheet")),
    (SHEETS, _("sheets")),
    (BUNCH, _("bunch")),
    (BUNDLE, _("bundle")),
    (CL, _("cl")),
    (CM, _("cm")),
    (THICKNESS, _("thick")),
    (DL, _("dl")),
    (CAN, _("can")),
    (CANS, _("cans")),
    (THIN, _("thin")),
    (BUCKET, _("bucket")),
    (AFEW, _("a few")),
    (AFEWSTALKS, _("a few stalks")),
    (TBS, _("tbs")),
    (TBSH, _("level tbs.")),
    (SOMETHING, _("something")),
    (POSSIBLY, _("possibly")),
    (EXTRA, _("extra")),
    (BOTTLE, _("bottle")),
    (BOTTLES, _("bottles")),
    (G, _("g")),
    (GLASS, _("glass")),
    (GLASSES, _("glasses")),
    (GREATCAN, _("great can")),
    (GREATBOTTLE, _("great bottle")),
    (GREATSHE, _("great she")),
    (GREAT, _("great")),
    (GREATHE, _("great he")),
    (GREATIT, _("great it")),
    (HALF, _("half")),
    (HANDFUL, _("handful")),
    (KG, _("kg")),
    (SMALLBUNCH, _("small bunch")),
    (SMALLCAN, _("small can")),
    (SMALLGLASS, _("small glass")),
    (SMALLHEAD, _("small head")),
    (SMALLBOTTLE, _("small bottle")),
    (SMALLSHE, _("small she")),
    (SMALL, _("small")),
    (SMALLHE, _("small he")),
    (SMALLIT, _("small it")),
    (TUBER, _("tuber")),
    (HEAD, _("head")),
    (BALL, _("ball")),
    (BALLS, _("balls")),
    (BALLS2, _("some balls")),
    (BOX, _("box")),
    (HEADS, _("heads")),
    (CORN, _("corn")),
    (LITER, _("liter")),
    (MGREATSHE, _("medium she")),
    (MGREATHE, _("medium he")),
    (MGREATIT, _("medium it")),
    (MORE, _("more")),
    (MG, _("mg")),
    (ML, _("ml")),
    (MSP, _("msp.")),
    (NB, _("n. B.")),
    (PAIR, _("pair")),
    (PACKAGE, _("package")),
    (PCK, _("pck.")),
    (PKT, _("pkt.")),
    (PLATE, _("plate")),
    (PORT, _("port.")),
    (PRISE, _("prise")),
    (PRISES, _("prises")),
    (RING, _("ring")),
    (RIB, _("rib")),
    (DISC, _("disc")),
    (DISCS, _("discs")),
    (SHOT, _("shot")),
    (TAPENADE, _("tapenade")),
    (SPLASH, _("splash")),
    (ROD, _("rod")),
    (STALK, _("stalk")),
    (STALKS, _("stalks")),
    (PIECE, _("piece")),
    (CULM, _("culm")),
    (CHALKBOARD, _("chalkboard")),
    (CHALKBOARDS, _("chalkboards")),
    (CUP2, _("cup2")),
    (CUPS, _("cups")),
    (PART, _("part")),
    (TL, _("tl.")),
    (HEAPEDTL, _("heaped tl.")),
    (LEVELTL, _("level tl.")),
    (POT, _("pot")),
    (DROP, _("drop")),
    (TUBE, _("tube")),
    (BAG2, _("bag2")),
    (LOTS, _("lots")),
    (FEW, _("few")),
    (ROOT, _("root")),
    (ROOTS, _("roots")),
    (CUBE, _("cube")),
    (TOE, _("toe")),
    (BRACE, _("brace")),
)